
$onEolCom

* ================input scalars=========================
* scalars used in 'sets' setting
$setlocal   Nt   96    !! number of time steps in a one day


Scalars 
* unit cost
    alpha_CHP   'cost coefficient of CHP, $/h'            /15/
    beta_CHP    'cost coefficient of CHP, $/MWhe'         /45/
    gamma_CHP   'cost coefficient of CHP, $/MWhth'        /20/
    c_CHP_RU    'unit cost of CHP reserve up, $/MWh'      /1/
    c_CHP_RD    'unit cost of CHP reserve down, $/MWh'    /1/
    c_BS        'unit operating cost of battery, $/MWh'   /40/
    c_wind      'unit operating cost of WT, $/MWh'        /10/
    c_price     'carbon price, $/ton'                     /50/
    c_inc       'unit incentive of DAC, $/ton'            /86/    
    
* CHP
    P_A     'electrical power at point A, MW'       /4.4/
    P_B     'electrical power at point B, MW'       /4.0/
    P_C     'electrical power at point C, MW'       /2.4/
    P_D     'electrical power at point D, MW'       /2.53/
    H_A     'thermal power at point A, MW'          /0/
    H_B     'thermal power at point B, MW'          /3.8/
    H_C     'thermal power at point C, MW'          /1.99/
    H_D     'thermal power at point D, MW'          /0/
    bigM_CHP    'big-M for CHP'                     /20/
    lambda_CHP  'proportion of low-pressure steam in CHP'       /0.399/
    
* battery storage
    eta_BS_ch       'charging efficiency of BS'              /0.95/
    eta_BS_dc       'discharging efficiency of BS'           /0.95/
    tau_BS          'coefficient of power and energy'        /0.4/
    upsilon_BS      'initial proportion of stored energy'    /0.2/
    
* DRCC
    eps_CHP         'confidence level of CHP'                /0.05/
    eps_BS          'confidence level of BS'                 /0.05/
    
* others
    delta_t         'dispatching interval, h'          /0.25/
    bigM_BS         'big-M of battery storage'         /5/ ;

Set
    t  'time step in one day'             /1*%Nt%/
    i  'equipment in IEHM'                /wind, CHP, BS, E_load, H_load/ ;
Alias (t,k,j);

* =========import data from Excel=========
* forecasting wind power  
Parameter P_wind_norm(t);   
$call 'del wind.gdx'
$call gdxxrw I=wind.xls O=wind.gdx par=P_wind_norm rdim=1 rng=sheet1!A1:B96
$gdxin wind.gdx
$loaddc P_wind_norm
$gdxin

Parameter P_load_norm(t);   
$call 'del P_load.gdx'
$call gdxxrw I=P_load.xls O=P_load.gdx par=P_load_norm rdim=1 rng=sheet1!A1:B96
$gdxin P_load.gdx
$loaddc P_load_norm
$gdxin

Parameter H_load_norm(t);   
$call 'del H_load.gdx'
$call gdxxrw I=H_load.xls O=H_load.gdx par=H_load_norm rdim=1 rng=sheet1!A1:B96
$gdxin H_load.gdx
$loaddc H_load_norm
$gdxin


Parameter   Cap(i)    'capacity of equipment'  !! MW, MW, MW, MW, MWh, MW, MWth
            / 'wind'    3.2   
              'CHP'     4.0
              'BS'      3.0
              'E_load'  4.64
              'H_load'  3.3 / ; 
    
Parameter   P_wind(t)    'forecasting power of wind farm, MW';
            P_wind(t) = P_wind_norm(t)*Cap('wind');

Parameter   P_load(t)   'actual electrical load, MW';
            P_load(t) = P_load_norm(t)*Cap('E_load');

Parameter   H_load(t)   'actual thermal load, MW';
            H_load(t) = P_load_norm(t)*Cap('H_load');
            
Parameter   K_CHP       'auxiliary parameter for CHP';
            K_CHP = sqrt((1-eps_CHP)/eps_CHP);
                        
Parameter   K_BS        'auxiliary parameter for BS';
            K_BS = sqrt((1-eps_BS)/eps_BS);
            
Parameter   mu_wind(t)  'mean of wind forecasting error';
            mu_wind(t) = 0;
            
Parameter   sigma_wind(t)   'variance of wind forecasting error';
            sigma_wind(t) = sqr(0.1*P_wind(t));
            
Parameter   cov_wind(t,k)   'covariance of wind forecasting error';
            cov_wind(t,k)$(ord(t)=ord(k)) = sigma_wind(t);
            cov_wind(t,k)$(ord(t)<>ord(k)) =  sqr(0.05)*P_wind(t)*P_wind(k);

Variables
* cost
    F       'total daily cost of IEHM'
    
* BS
    alpha2_ch(t)    'offsetting proportion of BS charging'
    W_BS_ch(t)      'auxiliary variable for BS charging'
    W_BS_dc(t)      'auxiliary variable for BS discharging'
    
* DRCC
    R_BS_E(t)      'auxiliary variable for BS'
    S_BS_E(t)      'auxiliary variable for BS'  ;

Positive variable
* CHP
    P_CHP(t)        'electrical power of CHP, MW'
    H_CHP(t)        'thermal power of CHP, MW'
    alpha1(t)       'offsetting proportion of CHP'
    RU_CHP(t)       'reserve up of CHP, MW'
    RD_CHP(t)       'reserve down of CHP, MW'
    
* battery storage
    P_BS_ch(t)      'charging power of BS'
    P_BS_dc(t)      'discharging power of BS'
    E_BS(t)         'electrical energy stored by BS'
    alpha2_dc(t)    'offsetting proportion of BS discharging'
    
* DRCC
    zeta_CHP(t)     'auxiliary variable for CHP'
    zeta_Res(t)     'auxiliary variable for CHP reserve capacity'
    zeta_BS_ch(t)   'auxiliary variable for BS'
    zeta_BS_dc(t)   'auxiliary variable for BS'
    u_BS_E(t)       'auxiliary variable for BS'
    v_BS_E(t)       'auxiliary variable for BS'
    aux_BS(t)
    aux_BS1(t);
    
Binary variable
* CHP    
    y_CHP(t)        'operating status of CHP'
        
* battery storage
    y_BS_ch(t)      'charging status of BS'
    y_BS_dc(t)      'discharging status of BS' ;
    
* =========================objective function========================
Equations
* Declaration 
obj_fun         'objective function' ;

* Definition
obj_fun     ..      F =e= sum(t, alpha_CHP+beta_CHP*P_CHP(t)+gamma_CHP*H_CHP(t)
                                +c_CHP_RU*RU_CHP(t)+c_CHP_RD*RD_CHP(t)
                                +c_wind*P_wind(t)+c_BS*(P_BS_ch(t)+P_BS_dc(t))
                                -c_BS*mu_wind(t)*(alpha2_ch(t)+alpha2_dc(t)))*delta_t ;

* =========================Constraints========================
Equations
* Decalaration of deterministic CHP constraints
Heat_bound_CHP(t)          'bounds of thermal power'
Ele_bound1_CHP(t)          'upper bound of electrical power'
Ele_bound2_CHP(t)          'lower bound of electrical power' ;

* Definition of deterministic CHP constraints
Heat_bound_CHP(t)      ..   H_CHP(t) =l= y_CHP(t)*H_B;
Ele_bound1_CHP(t)      ..   P_CHP(t)+RU_CHP(t) =l= y_CHP(t)*P_A;
Ele_bound2_CHP(t)      ..   P_CHP(t)-RD_CHP(t) =g= y_CHP(t)*P_C;

Equations
* Decalaration of CHP chance-constraints
Ele_heat_1(t)               'reformulated CC1 for the CHP operating region'
Ele_heat_2(t)               'reformulated CC2 for the CHP operating region'
Ele_heat_3(t)               'reformulated CC3 for the CHP operating region'
Ele_heat_SOC(t)             'SOC of reformulated CC for the CHP operating region' 

RU_bound_CHP(t)         'reserve capacity of CHP'
RD_bound_CHP(t)         'reserve capacity of CHP'
Reserve_CHP_SOC(t)      'reserve capacity of CHP' ;  

* Definition of CHP chance-constraints
Ele_heat_1(t)       ..      -alpha1(t)*mu_wind(t)+P_CHP(t)-P_A-(P_A-P_B)/(H_A-H_B)
                                *(H_CHP(t)-H_A)+K_CHP*zeta_CHP(t) =l= 0;
Ele_heat_2(t)       ..      alpha1(t)*mu_wind(t)-P_CHP(t)+P_B+(P_B-P_C)/(H_B-H_C)
                                *(H_CHP(t)-H_C)+K_CHP*zeta_CHP(t) =l= (1-y_CHP(t))*bigM_CHP;
Ele_heat_3(t)       ..      alpha1(t)*mu_wind(t)-P_CHP(t)+P_C+(P_C-P_D)/(H_C-H_D)
                                *(H_CHP(t)-H_D)+K_CHP*zeta_CHP(t) =l= (1-y_CHP(t))*bigM_CHP;
Ele_heat_SOC(t)     ..      alpha1(t)*sigma_wind(t) =l= zeta_CHP(t);

RU_bound_CHP(t)     ..  -alpha1(t)*mu_wind(t)+K_CHP*zeta_Res(t) =l= RU_CHP(t);
RD_bound_CHP(t)     ..  alpha1(t)*mu_wind(t)+K_CHP*zeta_Res(t) =l= RD_CHP(t);
Reserve_CHP_SOC(t)  ..  alpha1(t)*sigma_wind(t) =l= zeta_Res(t);

Equations
* Decalaration of power balance
Ele_balance(t)          'electrical power balance of IEHM'
Heat_balance(t)         'thermal power balance of IEHM';

* Definition of power balance
Ele_balance(t)      ..  P_wind(t)+P_CHP(t)+P_BS_dc(t) =e= P_load(t)+P_BS_ch(t);
Heat_balance(t)     ..  H_CHP(t) =e= H_load(t);

Equations
* Decalaration of battery storage chance-constraints
Ele_BS_charge_1(t)          'CC of BS charging power'
Ele_BS_charge_2(t)          'CC of BS charging power'
Ele_BS_charge_3(t)          'CC of BS charging power'

Ele_BS_discharge_1(t)       'CC of BS discharging power'
Ele_BS_discharge_2(t)       'CC of BS discharging power'
Ele_BS_discharge_3(t)       'CC of BS discharging power'

Ele_BS_energy_1(t)          'CC of BS energy'
Ele_BS_energy_2          'CC of BS energy'
Ele_BS_energy_3(t)          'CC of BS energy'
Ele_BS_energy_4(t)          'CC of BS energy'
Ele_BS_energy_5(t)          'CC of BS energy'
Ele_BS_energy_6(t)          'CC of BS energy'

Ele_BS_energy_ini           'initial energy level of BS'
Status_BS(t)                'single status of BS at each time' ;

* Definition of battery storage chance-constraints
Ele_BS_charge_1(t)      ..  -alpha2_ch(t)*mu_wind(t)+P_BS_ch(t)+K_BS*zeta_BS_ch(t)
                                =l= y_BS_ch(t)*tau_BS*Cap('BS');
Ele_BS_charge_2(t)      ..  alpha2_ch(t)*mu_wind(t)-P_BS_ch(t)+K_BS*zeta_BS_ch(t) =l= 0;
Ele_BS_charge_3(t)      ..  -alpha2_ch(t)*sigma_wind(t) =l= zeta_BS_ch(t);

Ele_BS_discharge_1(t)   ..  -alpha2_dc(t)*mu_wind(t)+P_BS_dc(t)+K_BS*zeta_BS_dc(t)
                                =l= y_BS_dc(t)*tau_BS*Cap('BS');
Ele_BS_discharge_2(t)   ..  alpha2_dc(t)*mu_wind(t)-P_BS_dc(t)+K_BS*zeta_BS_dc(t) =l= 0;
Ele_BS_discharge_3(t)   ..  alpha2_dc(t)*sigma_wind(t) =l= zeta_BS_dc(t);

Ele_BS_energy_1(t)      ..  sqr(u_BS_E(t))+sum(k$(ord(k)<=ord(t)), sqr(aux_BS(k)))
                                =l= sqr(aux_BS1(t));
Ele_BS_energy_2(t)      ..  S_BS_E(t) =l= u_BS_E(t)+v_BS_E(t);
Ele_BS_energy_3(t)      ..  -S_BS_E(t) =l= u_BS_E(t)+v_BS_E(t);
Ele_BS_energy_4(t)      ..  aux_BS1(t) =e= (Cap('BS')/2-v_BS_E(t))*sqrt(eps_BS);
Ele_BS_energy_5(t)      ..  S_BS_E(t) =e= E_BS('1')+sum(k$(ord(k)<=ord(t)), (P_BS_ch(k)*eta_BS_ch
                                -P_BS_ch(k)/eta_BS_dc)*delta_t)-Cap('BS')/2;
Ele_BS_energy_6(t)      ..  aux_BS(t) =e= sigma_wind(t)*(-alpha2_ch(t)*eta_BS_ch+alpha2_dc(t)/eta_BS_dc)*delta_t ;

aux_BS.lo(t) = 0;
aux_BS1.lo(t) = 0;

Ele_BS_energy_ini       ..  E_BS('1') =e= upsilon_BS*Cap('BS');
Status_BS(t)            ..  y_BS_ch(t)+y_BS_dc(t) =l= 1;

Equations
* Declaration of apportionment ratios
Appor_one(t)            'sum of apportionment ratios equals one'
Appor_bigM_ch1(t)       'big-M for apportionment of BS charging'
Appor_bigM_ch21(t)      'big-M for apportionment of BS charging'
Appor_bigM_ch22(t)      'big-M for apportionment of BS charging'
Appor_bigM_ch31(t)      'big-M for apportionment of BS charging'
Appor_bigM_ch32(t)      'big-M for apportionment of BS charging'
Appor_bigM_dc1(t)       'big-M for apportionment of BS discharging'
Appor_bigM_dc21(t)      'big-M for apportionment of BS discharging'
Appor_bigM_dc22(t)      'big-M for apportionment of BS discharging'
Appor_bigM_dc31(t)      'big-M for apportionment of BS discharging'
Appor_bigM_dc32(t)      'big-M for apportionment of BS discharging'
Appor_bound1(t)         'bound of alpha1'
Appor_bound21_ch(t)     'bound of alpha2_ch'
Appor_bound22_ch(t)     'bound of alpha2_ch'
Appor_bound2_dc(t)      'bound of alpha2_dc' ;

* Definition of apportionment ratios
Appor_one(t)            ..    alpha1(t)+alpha2_dc(t)-alpha2_ch(t) =e= 1;
Appor_bigM_ch1(t)       ..    alpha2_ch(t) =e= W_BS_ch(t);
Appor_bigM_ch21(t)      ..    W_BS_ch(t) =g= -bigM_BS*y_BS_ch(t);
Appor_bigM_ch22(t)      ..    W_BS_ch(t) =l= bigM_BS*y_BS_ch(t);
Appor_bigM_ch31(t)      ..    W_BS_ch(t) =g= alpha2_ch(t)+(y_BS_ch(t)-1)*bigM_BS;
Appor_bigM_ch32(t)      ..    W_BS_ch(t) =l= alpha2_ch(t)+(1-y_BS_ch(t))*bigM_BS;
Appor_bigM_dc1(t)       ..    alpha2_dc(t) =e= W_BS_dc(t);
Appor_bigM_dc21(t)      ..    W_BS_dc(t) =g= -bigM_BS*y_BS_dc(t);
Appor_bigM_dc22(t)      ..    W_BS_dc(t) =l= bigM_BS*y_BS_dc(t);
Appor_bigM_dc31(t)      ..    W_BS_dc(t) =g= alpha2_dc(t)+(y_BS_dc(t)-1)*bigM_BS;
Appor_bigM_dc32(t)      ..    W_BS_dc(t) =l= alpha2_dc(t)+(1-y_BS_dc(t))*bigM_BS;
Appor_bound1(t)         ..    alpha1(t) =l= 1;
Appor_bound21_ch(t)     ..    alpha2_ch(t) =l= 0;
Appor_bound22_ch(t)     ..    alpha2_ch(t) =g= -1;
Appor_bound2_dc(t)      ..    alpha2_dc(t) =l= 1;

* =======================Model definition==================
model m
        /obj_fun,
        
         Heat_bound_CHP, Ele_bound1_CHP, Ele_bound2_CHP,
         Ele_heat_1, Ele_heat_2, Ele_heat_3, Ele_heat_SOC,
         RU_bound_CHP, RD_bound_CHP, Reserve_CHP_SOC,
                  
         Ele_balance, Heat_balance,
         
         Ele_BS_charge_1, Ele_BS_charge_2, Ele_BS_charge_3, 
         Ele_BS_discharge_1, Ele_BS_discharge_2, Ele_BS_discharge_3,
         Ele_BS_energy_1,  Ele_BS_energy_2, Ele_BS_energy_3, Ele_BS_energy_ini, Status_BS,
         Ele_BS_energy_4,  Ele_BS_energy_5, Ele_BS_energy_6, 
         
         Appor_one, Appor_bigM_ch1, Appor_bigM_ch21, Appor_bigM_ch22, Appor_bigM_ch31,
         Appor_bigM_ch32, Appor_bigM_dc1, Appor_bigM_dc21, Appor_bigM_dc22,
         Appor_bigM_dc31, Appor_bigM_dc32, Appor_bound1, Appor_bound21_ch,
         Appor_bound22_ch, Appor_bound2_dc
        /;
           

option MIQCP = mosek;         
option reslim = 1000;
         
solve m minimizing F using MIQCP;


